from youtube_autonomous.elements.validator.element_parameter_validator import ElementParameterValidator
from youtube_autonomous.segments.enums import SegmentType, EnhancementType
from youtube_autonomous.segments.builder.ai import create_ai_narration
from yta_general_utils.programming.parameter_obtainer import ParameterObtainer
from typing import Union


class ElementBuilder:
    # TODO: Maybe __init__ that returns the corresponding subclass
    # according to the provided 'type' (?)
    # https://stackoverflow.com/questions/9187388/possible-to-prevent-init-from-being-called

    @staticmethod
    def get_subclasses():
        from youtube_autonomous.elements.builder.ai_image_element_builder import AIImageElementBuilder
        from youtube_autonomous.elements.builder.image_element_builder import ImageElementBuilder
        from youtube_autonomous.elements.builder.ai_video_element_builder import AIVideoElementBuilder
        from youtube_autonomous.elements.builder.video_element_builder import VideoElementBuilder
        from youtube_autonomous.elements.builder.custom_stock_element_builder import CustomStockElementBuilder
        from youtube_autonomous.elements.builder.stock_element_builder import StockElementBuilder
        from youtube_autonomous.elements.builder.meme_element_builder import MemeElementBuilder
        from youtube_autonomous.elements.builder.sound_element_builder import SoundElementBuilder
        from youtube_autonomous.elements.builder.youtube_video_element_builder import YoutubeVideoElementBuilder
        from youtube_autonomous.elements.builder.text_element_builder import TextElementBuilder
        from youtube_autonomous.elements.builder.premade_element_builder import PremadeElementBuilder
        from youtube_autonomous.elements.builder.effect_element_builder import EffectElementBuilder
        from youtube_autonomous.elements.builder.greenscreen_element_builder import GreenscreenElementBuilder

        return [
            AIImageElementBuilder,
            AIVideoElementBuilder,
            ImageElementBuilder,
            VideoElementBuilder,
            CustomStockElementBuilder,
            StockElementBuilder,
            MemeElementBuilder,
            SoundElementBuilder,
            YoutubeVideoElementBuilder,
            TextElementBuilder,
            PremadeElementBuilder,
            EffectElementBuilder,
            GreenscreenElementBuilder
        ]

    @staticmethod
    def get_subclasses_as_str():
        return ', '.join(ElementBuilder.get_subclasses())
    
    @staticmethod
    def get_subclass_by_type(type: Union[SegmentType, EnhancementType, str]):
        type = ElementParameterValidator.validate_segment_or_enhancement_type(type)

        from youtube_autonomous.elements.builder.ai_image_element_builder import AIImageElementBuilder
        from youtube_autonomous.elements.builder.image_element_builder import ImageElementBuilder
        from youtube_autonomous.elements.builder.ai_video_element_builder import AIVideoElementBuilder
        from youtube_autonomous.elements.builder.video_element_builder import VideoElementBuilder
        from youtube_autonomous.elements.builder.custom_stock_element_builder import CustomStockElementBuilder
        from youtube_autonomous.elements.builder.stock_element_builder import StockElementBuilder
        from youtube_autonomous.elements.builder.meme_element_builder import MemeElementBuilder
        from youtube_autonomous.elements.builder.sound_element_builder import SoundElementBuilder
        from youtube_autonomous.elements.builder.youtube_video_element_builder import YoutubeVideoElementBuilder
        from youtube_autonomous.elements.builder.text_element_builder import TextElementBuilder
        from youtube_autonomous.elements.builder.premade_element_builder import PremadeElementBuilder
        from youtube_autonomous.elements.builder.effect_element_builder import EffectElementBuilder
        from youtube_autonomous.elements.builder.greenscreen_element_builder import GreenscreenElementBuilder

        if type in [SegmentType.MEME, EnhancementType.MEME]:
            return MemeElementBuilder
        elif type in [SegmentType.AI_IMAGE, EnhancementType]:
            return AIImageElementBuilder
        elif type in [SegmentType.AI_VIDEO, EnhancementType.AI_VIDEO]:
            return AIVideoElementBuilder
        elif type in [SegmentType.IMAGE, EnhancementType.IMAGE]:
            return ImageElementBuilder
        elif type in [SegmentType.VIDEO, EnhancementType.VIDEO]:
            return VideoElementBuilder
        elif type in [SegmentType.STOCK, EnhancementType.STOCK]:
            return StockElementBuilder
        elif type in [SegmentType.CUSTOM_STOCK, EnhancementType.CUSTOM_STOCK]:
            return CustomStockElementBuilder
        elif type in [SegmentType.SOUND, EnhancementType.SOUND]:
            return SoundElementBuilder
        elif type in [SegmentType.YOUTUBE_VIDEO, EnhancementType.YOUTUBE_VIDEO]:
            return YoutubeVideoElementBuilder
        elif type in [SegmentType.TEXT, EnhancementType.TEXT]:
            return TextElementBuilder
        elif type in [SegmentType.PREMADE, EnhancementType.PREMADE]:
            return PremadeElementBuilder
        elif type in [EnhancementType.EFFECT]:
            return EffectElementBuilder
        elif type in [EnhancementType.GREENSCREEN]:
            return GreenscreenElementBuilder
    
    @classmethod
    def build_narration(cls, text: str, output_filename: str):
        """
        Generates a narration file that narrates the 'text' provided and
        is stored locally as 'output_filename'. If 'text' or 
        'output_filename' fields are not provided it will raise an 
        Exception.
        """
        ElementParameterValidator.validate_string_mandatory_parameter('text', text)
        ElementParameterValidator.validate_string_mandatory_parameter('output_filename', output_filename)

        return create_ai_narration(text, output_filename = output_filename)
    
    @classmethod
    def extract_extra_params(cls, element, method, params_to_ignore: list[str] = ['self', 'cls', 'args', 'kwargs']):
        """
        Returns an object containing 'mandatory' and 'optional' fields that
        are the parameters extracted from the provided 'element' 'extra_data'
        field and according to the also provided 'method' (the parameters in
        the signature of that 'method' are the ones that are extracted).

        This method raises an exception if some of the 'mandatory' fields are
        not given in the 'extra_data' field.
        """
        parameters_needed = ParameterObtainer.get_parameters_from_method(method, params_to_ignore)  
        parameters = {}

        if isinstance(element, dict):
            extra_params = element.get('extra_params')
        else:
            extra_params = element.extra_params

        if not extra_params and not isinstance(extra_params, dict):
            raise Exception('No "extra_params" found.')

        missing_mandatory_parameters = []
        for mandatory_parameter in parameters_needed['mandatory']:
            if not mandatory_parameter in extra_params:
                missing_mandatory_parameters.append(mandatory_parameter)
            else:
                parameters[mandatory_parameter] = extra_params[mandatory_parameter]

        if len(missing_mandatory_parameters) > 0:
            missing_params_str = ', '.join(missing_mandatory_parameters)
            raise Exception(f'The extra parameters "{missing_params_str}" are missing on the provided element and they are needed to build it.')

        for optional_parameter in parameters_needed['optional']:
            if optional_parameter in extra_params:
                parameters[optional_parameter] = extra_params[optional_parameter]

        return parameters
    
    @classmethod
    def handle_narration_from_segment(cls, segment: dict):
        pass


from youtube_autonomous.database.database_handler import DatabaseHandler
from youtube_autonomous.shortcodes.shortcode_parser import ShortcodeParser
from youtube_autonomous.elements.builder.element_builder import ElementBuilder
from youtube_autonomous.enums import StringDuration
from youtube_autonomous.segments.enhancement.edition_manual.edition_manual import EditionManual
from youtube_autonomous.elements.enhancement import Enhancement
from youtube_autonomous.elements.rules.effect_element_rules import ElementRules
from youtube_autonomous.elements.rules.rules_checker import RulesChecker
from youtube_autonomous.segments.builder.config import DEFAULT_SEGMENT_PARTS_FOLDER
from youtube_autonomous.segments.enums import EnhancementMode, EnhancementType
from youtube_autonomous.elements.builder.effect_element_builder import EffectElementBuilder
from yta_multimedia.audio.voice.transcription.stt.whisper import get_transcription_with_timestamps
from yta_multimedia.audio.sound.generation.sound_generator import SoundGenerator
from yta_general_utils.file import copy_file
from yta_general_utils.file.filename import get_file_extension
from yta_general_utils.temp import get_temp_filename
from yta_general_utils.logger import print_completed, print_in_progress
from moviepy.editor import AudioFileClip, VideoFileClip, CompositeVideoClip, concatenate_videoclips, CompositeAudioClip


class Segment:
    project_id: int
    index: int
    status = ''
    type = ''

    audio_filename: str = ''
    audio_clip = None
    video_filename: str = ''
    video_clip = None
    full_filename: str = ''
    full_clip = None

    audio_narration_filename: str
    voice: str
    narration_text: str

    text: str

    keywords: str

    duration: float = None

    filename: str
    url: str

    music: str

    enhancements: list

    extra_params: dict

    narration_text_sanitized_without_shortcodes: str
    narration_text_with_simplified_shortcodes: str
    narration_text_sanitized: str
    transcription = None
    shortcodes = None
    created_at = None

    def set_audio_filename(self, audio_filename: str):
        self.audio_filename = audio_filename

        if audio_filename:
            self.audio_clip = AudioFileClip(self.audio_filename)
            self.database_handler.update_project_segment_field(self.project_id, self.index, 'audio_filename', self.audio_filename)

    def set_video_filename(self, video_filename: str):
        self.video_filename = video_filename

        if video_filename:
            self.video_clip = VideoFileClip(self.video_filename)
            self.database_handler.update_project_segment_field(self.project_id, self.index, 'video_filename', self.video_filename)

    def set_full_filename(self, full_filename: str):
        self.full_filename = full_filename

        if full_filename:
            self.full_clip = VideoFileClip(self.full_filename)
            self.database_handler.update_project_segment_field(self.project_id, self.index, 'full_filename', self.full_filename)

    def set_transcription(self, transcription):
        self.transcription = transcription

        if transcription:
            self.database_handler.update_project_segment_field(self.project_id, self.index, 'transcription', transcription)

    def set_shortcodes(self, shortcodes):
        self.shortcodes = shortcodes

        if shortcodes:
            self.database_handler.update_project_segment_field(self.project_id, self.index, 'shortcodes', shortcodes)

    def set_as_finished(self):
        self.database_handler.update_project_segment_status(self.project_id, self.index, 'finished')

    def __init__(self, project_id, index: int, data: dict):
        self.project_id = project_id
        self.index = index

        for key in data:
            setattr(self, key, data[key])

        if self.audio_filename:
            self.audio_clip = AudioFileClip(self.audio_filename)
        if self.video_filename:
            self.video_clip = VideoFileClip(self.video_filename)
        if self.full_filename:
            self.full_clip = VideoFileClip(self.full_filename)

        self.rules = ElementRules.get_subclass_by_type(data['type'])()
        self.builder = ElementBuilder.get_subclass_by_type(data['type'])()
        self.rules_checker = RulesChecker(self.rules)

        self.database_handler = DatabaseHandler()
        # TODO: Set shortcode tags please (read Notion)
        self.shortcode_parser = ShortcodeParser([])

    # TODO: This below has been moved because inheritance is working strangely

    def create_segment_file(self, filename: str):
        """
        Creates a filename within the definitive segments folder
        to keep the generated file locally to recover it in the
        next project execution if something goes wrong. The 
        definitive filename will be built using the provided
        'filename' and adding some more information in the name.

        This method will generate a temporary filename that uses
        the current segment index in its name and is placed in 
        segment parts folder.

        This method returns the final filename created.
        """
        if not filename:
            raise Exception('No "filename" provided.')

        temp_filename = get_temp_filename(filename)

        return f'{DEFAULT_SEGMENT_PARTS_FOLDER}/segment_{self.index}_{temp_filename}'
    
    def create_narration(self):
        """
        Creates the audio narration (if needed) by generating an AI audio
        narration with provided 'voice' and 'audio_narration_text'
        parameters or by using the 'audio_narration_filename'.

        This method will set the 'audio_filename' to be able to build the
        audio clip in a near future.
        """
        if self.audio_narration_filename:
            segment_part_filename = self.create_segment_file(f'narration.{get_file_extension(self.audio_narration_filename)}')
            copy_file(self.audio_narration_filename, segment_part_filename)
            print_completed('Original voice narration file copied to segment parts folder')
            self.audio_narration_filename = segment_part_filename
            self.set_audio_filename(segment_part_filename)
        else:
            segment_part_filename = self.create_segment_file('narration.wav')
            # TODO: Voice parameter need to change
            self.set_audio_filename(self.builder.build_narration(self.narration_text_sanitized_without_shortcodes, output_filename = segment_part_filename))
            print_completed('Voice narration created successfully')

    def create_transcription(self):
        """
        Creates the transcription of the generated audio narration
        that would be stored in 'self.audio_filename'.
        
        This method returns a words array containing, for each word,
        a 'text', 'start' and 'end' field to be able to use the 
        transcription timestamps.
        """
        transcription = get_transcription_with_timestamps(self.audio_filename, initial_prompt = self.narration_text_sanitized_without_shortcodes)

        # We join all words together as we don't need segments, just words with time
        words = []
        for transcription_segment in transcription['segments']:
            words += transcription_segment['words']
        self.set_transcription(words)

    # TODO: This above has been moved because inheritance is working strangely

    # TODO: This below has to be in the common Object from wich Segment
    # and Enhancement will inherit
    def build_step_1_create_narration(self):
        # 1. Generate narration if needed
        self.create_narration()

    def build_step_2_create_transcription(self):
        # 2. Generate narration transcription
        self.create_transcription()

    def build_step_2_extract_user_shortcodes(self):
        # 2. Extract manually written shortcodes in 'narration_text'
        self.shortcode_parser.parse(self.narration_text)
        self.shortcodes = self.shortcode_parser.shortcodes
        # TODO: Turn shortcodes into enhancements
        # TODO: I'm doing nothing with these enhancements
        # TODO: Maybe store in database (?)
        enhancements = [shortcode.to_enhancement_element(self.transcription) for shortcode in self.shortcodes]

    def build_step_3_apply_edition_manual_shortcodes(self):
        # 3. Apply shortcodes from Edition Manual and 'narration_text'
        # TODO: Maybe store in database (?)
        self.apply_edition_manual()

    def build_step_4_build_base_content(self):
        # 4. Build base video
        self.video_clip = self.builder.build_from_segment(self)
        filename = self.create_segment_file('video.mp4')
        self.video_clip.write_videofile(filename)
        self.set_video_filename(filename)

    def build_step_5_update_enhancements_duration(self):
        for index, enhancement in enumerate(self.enhancements):
            if isinstance(enhancement, dict):
                self.enhancements[index] = Enhancement(self.project_id, self.index, self, index + 100, enhancement)
        # TODO: Convert dict enhancements to enhancement objects
        # 5. Update enhancements duration according to base video
        # TODO: What if 'start' is after the end of the base video (?)
        for index, enhancement in enumerate(self.enhancements):
            # With this condition we avoid the None enhancements that are 
            # generated due to the gap between the start indexes and the
            # ones we generate with +100 because of edition manual
            if enhancement is None:
                continue

            if enhancement.duration == StringDuration.SEGMENT_DURATION.name:
                enhancement.duration = self.video_clip.duration

            if enhancement.start >= self.video_clip.duration:
                raise Exception('The enhancement start moment is after the video_clip.duration')
            
            end = enhancement.start + enhancement.duration
            if end > self.video_clip.duration:
                enhancement.duration = self.video_clip.duration - enhancement.start

            if enhancement.mode == EnhancementMode.INLINE.value:
                # If inline and start is after this moment, increase its
                # start as we are enlarging the video when applying this
                # inline enhancement
                for i in range(index + 1, len(self.enhancements)):
                    if self.enhancements[i] is None:
                        continue
                    
                    if self.enhancements[i].start >= enhancement.start:
                        self.enhancements[i].start += enhancement.duration
            
    def build_step_6_build_and_apply_enhancements(self):
        """
        Builds the different Enhancements according to their
        type and parameters and applies them into the main
        video.
        """
        # I obtain them sorted to start from the first one to apply them
        for enhancement in sorted(self.enhancements, key = lambda enhancement: enhancement.start):
            if enhancement is None:
                # TODO: This should not exist if I don't have 100 None segments
                # because of my +100 index identifying strategy
                continue

            enhancement_clip = enhancement.build()
            # TODO: Combine with our segment according to type

            end = enhancement.start + enhancement.duration
            if enhancement.mode == EnhancementMode.INLINE.value:
                if enhancement.type in [EnhancementType.EFFECT.value, EnhancementType.EFFECT]:
                    parameters_to_ignore = ['self', 'cls', 'args', 'kwargs', 'video', 'duration']
                    parameters_not_from_extra = ['duration']
                    parameters = EffectElementBuilder.get_building_parameters(enhancement, parameters_to_ignore, parameters_not_from_extra)

                    enhancement_clip = enhancement_clip.apply(self.video_clip.subclip(enhancement.start, end), **parameters)

                if enhancement.start > 0:
                    self.video_clip = concatenate_videoclips([
                        self.video_clip.subclip(0, enhancement.start),
                        enhancement_clip,
                        self.video_clip.subclip(enhancement.start, self.video_clip.duration)
                    ])
                elif enhancement.start == 0:
                    self.video_clip = concatenate_videoclips([
                        enhancement_clip,
                        self.video_clip
                    ])
                # TODO: What about start == clip.duration (?)
            elif enhancement.mode == EnhancementMode.OVERLAY.value:
                compound_clip = CompositeVideoClip([
                    self.video_clip.subclip(enhancement.start, end),
                    enhancement_clip
                ])

                if enhancement.start == 0 and enhancement.duration == self.video_clip.duration:
                    # TODO: Improve this
                    self.video_clip = compound_clip
                elif enhancement.start == 0:
                    self.video_clip = concatenate_videoclips([
                        compound_clip,
                        self.video_clip.subclip(end, self.video_clip.duration)
                    ])
                elif end == self.video_clip.duration:
                    self.video_clip = concatenate_videoclips([
                        self.video_clip.subclip(0, enhancement.start),
                        compound_clip
                    ])
                else:
                    self.video_clip = concatenate_videoclips([
                        self.video_clip.subclip(0, enhancement.start),
                        compound_clip,
                        self.video_clip.subclip(end, self.video_clip.duration)
                    ])
                # TODO: Could happen that duration is longer than self.video_clip.duration (?)
            elif enhancement.mode == EnhancementMode.REPLACE.value:
                # I first build the special ones
                if enhancement.type in [EnhancementType.EFFECT.value, EnhancementType.EFFECT]:
                    parameters_to_ignore = ['self', 'cls', 'args', 'kwargs', 'video', 'duration']
                    parameters_not_from_extra = ['duration']
                    parameters = EffectElementBuilder.get_building_parameters(enhancement, parameters_to_ignore, parameters_not_from_extra)

                    enhancement_clip = enhancement_clip.apply(self.video_clip.subclip(enhancement.start, end), **parameters)
                elif enhancement.type in [EnhancementType.GREENSCREEN.value, EnhancementType.GREENSCREEN]:
                    # TODO: Send parameters (I don't know how actually)
                    enhancement_clip = enhancement_clip.from_video_to_video(self.video_clip.subclip(enhancement.start, end))

                # Then just push the clip when needed
                if enhancement.start == 0 and enhancement.duration == self.video_clip.duration:
                    self.video_clip = enhancement_clip
                elif enhancement.start == 0:
                    self.video_clip = concatenate_videoclips([
                        enhancement_clip,
                        self.video_clip.subclip(end, self.video_clip.duration)
                    ])
                elif enhancement.duration == self.video_clip.duration:
                    self.video_clip = concatenate_videoclips([
                        self.video_clip.subclip(0, enhancement.start),
                        enhancement_clip
                    ])
                else:
                    self.video_clip = concatenate_videoclips([
                        self.video_clip.subclip(0, enhancement.start),
                        enhancement_clip,
                        self.video_clip.subclip(end, self.video_clip.duration)
                    ])

    def build(self):
        if self.rules_checker.should_build_narration_rule(self):
            if not self.audio_filename:
                self.build_step_1_create_narration()

            # We are forcing duration here
            self.duration = self.audio_clip.duration

            if not self.transcription:
                self.build_step_2_create_transcription()

            # TODO: Handle this (maybe status (?))
            self.build_step_2_extract_user_shortcodes()

            self.build_step_3_apply_edition_manual_shortcodes()

        # TODO: What about duration here that is not set (?)
        if self.audio_clip:
            self.duration = self.audio_clip.duration

        if not self.video_clip:
            print_in_progress('Building base content step 4')
            self.build_step_4_build_base_content()
            print_completed('Base content built in step 4')

        # This would be the real duration that could be not the
        # one in the database as this is in real time and it is
        # not updated in the database
        self.duration = self.video_clip.duration

        # Make the base video to apply enhancements on it
        if self.audio_clip:
            # If we have another audio clip we should want to put
            # them togeter, not to replace it
            audio = self.audio_clip
            if self.video_clip.audio:
                # We have another audio, we need to put them together
                audio = CompositeAudioClip([
                    self.video_clip.audio,
                    audio
                ])
                
            # TODO: Should we replace self.audio_clip (?)
            self.video_clip = self.video_clip.set_audio(audio)
        elif not self.video_clip.audio:
            # Premade and other type of video_clip can have audio
            # by themselves, but another type of videos cannot, so
            # we need to put silence audio to have an audio track
            # or we will have issues when concatenating with ffmpeg
            # only in those without audio
            self.video_clip = self.video_clip.set_audio(SoundGenerator().create_silence_audio(self.video_clip.duration))

        self.build_step_5_update_enhancements_duration()
        # TODO: By now I'm omitting this part
        self.build_step_6_build_and_apply_enhancements()

        # TODO: Build the final video
        self.full_clip = self.video_clip
        filename = self.create_segment_file('definitivo.mp4')
        self.full_clip.write_videofile(filename)
        self.set_full_filename(filename)

        self.set_as_finished()


    def apply_edition_manual(self):
        # TODO: I need to dynamically get the edition manual from somewhere
        # By now I'm forcing it
        test_edition_manual = 'C:/Users/dania/Desktop/PROYECTOS/youtube-autonomous/youtube_autonomous/segments/enhancement/edition_manual/example.json'
        edition_manual = EditionManual.init_from_file(test_edition_manual)
        dict_enhancements_found = edition_manual.apply(self.transcription)
        # Turn dict enhancements found into Enhancement objects
        for index, dict_enhancement_found in enumerate(dict_enhancements_found):
            # TODO: What do we do with 'index' here (?)
            index = 100 + index
            self.enhancements.append(Enhancement(self.project_id, self.index, self, index, dict_enhancement_found))
        # TODO: Some enhancements could be incompatible due to collisions
        # or things like that

        # TODO: What about this enhancements (?)



from youtube_autonomous.segments.enums import SegmentField, EnhancementField, SegmentType, ShortcodeType, EnhancementType, EnhancementMode, ShortcodeStringDuration, EnhancementStringDuration, SegmentStringDuration, EnhancementStringDuration, SegmentMode, ShortcodeMode, ShortcodeField, SegmentStart, EnhancementStart, ShortcodeStart
from youtube_autonomous.elements.validator.element_parameter_validator import ElementParameterValidator
from youtube_autonomous.config import Configuration
from yta_general_utils.checker.type import variable_is_positive_number
from yta_general_utils.programming.enum import YTAEnum as Enum
from typing import Union


# TODO: Move this enum to enums file
class Component(Enum):
    SEGMENT = 'segment'
    ENHANCEMENT = 'enhancement'
    SHORTCODE = 'shortcode'

class ElementValidator:
    """
    Class to validate the segment o enhancement element fields and
    their values.
    """
    @staticmethod
    def validate_segment_fields(segment: dict):
        """
        Validates that the 'segment' dictionary provided has all the
        required segment fields and only those ones and raises an
        Exception if more parameters or less parameters exist.

        This method returns the 'segment' provided if everything is
        ok.
        """
        return ElementValidator.validate_fields(Component.SEGMENT, segment)

    @staticmethod
    def validate_enhancement_fields(enhancement: dict):
        """
        Validates that the 'enhancement' dictionary provided has 
        all the required enhancement fields and only those ones and 
        raises an Exception if more parameters or less parameters
        exist.

        This method returns the 'enhancement' provided if everything is
        ok.
        """
        return ElementValidator.validate_fields(Component.ENHANCEMENT, enhancement)

    @staticmethod
    def validate_shortcode_fields(shortcode: dict):
        """
        Validates that the 'shortcode' dictionary provided has 
        all the required shortcode fields and only those ones and 
        raises an Exception if more parameters or less parameters
        exist.

        This method returns the 'shortcode' provided if everything is
        ok.
        """
        return ElementValidator.validate_fields(Component.SEGMENT, shortcode)

    @staticmethod
    def validate_fields(component: Component, element: dict):
        if component == Component.SEGMENT:
            accepted_fields = SegmentField.get_all_values()
        elif component == Component.ENHANCEMENT:
            accepted_fields = EnhancementField.get_all_values()
        elif component == Component.SHORTCODE:
            accepted_fields = ShortcodeField.get_all_values()

        accepted_fields_str = ', '.join(accepted_fields)
        unaccepted_fields = [key for key in element.keys() if key not in accepted_fields]
        unaccepted_fields_str = ', '.join(unaccepted_fields)
        missing_fields = [field for field in accepted_fields if field not in element]
        missing_fields_str = ', '.join(missing_fields)

        if missing_fields:
            raise Exception(f'The next fields are mandatory and were not found in the element: "{missing_fields_str}". The mandatory fields are: "{accepted_fields_str}".')

        if unaccepted_fields:
            raise Exception(f'The next fields are not accepted in the provided element by our system: "{unaccepted_fields_str}". The ones accepted are these: "{accepted_fields_str}".')
        
        return element
    
    @staticmethod
    def validate_segment_mode_field(mode: Union[SegmentMode, str, None]):
        """
        Validates if the provided 'mode' parameter is a valid parameter
        for a segment object. It should be a SegmentMode enum or one of
        its string values to be accepted.
        
        This method will raise an Exception if not valid or will return
        it as it is, or as a SegmentMode if one of its valid string
        values provided.
        """
        return ElementValidator.validate_mode_field(Component.SEGMENT, mode)

    @staticmethod
    def validate_enhancement_mode_field(mode: Union[EnhancementMode, str, None]):
        """
        Validates if the provided 'mode' parameter is a valid parameter
        for an enhancement object. It should be a EnhancementMode enum
        or one of its string values to be accepted.
        
        This method will raise an Exception if not valid or will return
        it as it is, or as a EnhancementMode if one of its valid string
        values provided.
        """
        return ElementValidator.validate_mode_field(Component.ENHANCEMENT, mode)
    
    @staticmethod
    def validate_shortcode_mode_field(mode: Union[ShortcodeMode, str, None]):
        """
        Validates if the provided 'mode' parameter is a valid parameter
        for a shortcode object. It should be a ShortcodeMode enum
        or one of its string values to be accepted.
        
        This method will raise an Exception if not valid or will return
        it as it is, or as a ShortcodeMode if one of its valid string
        values provided.
        """
        return ElementValidator.validate_mode_field(Component.SHORTCODE, mode)
    
    @staticmethod
    def validate_mode_field(component: Component, mode: Union[SegmentMode, EnhancementMode, ShortcodeMode, str, None]):
        if mode is not None:
            if component == Component.SEGMENT:
                mode = SegmentMode.to_enum(mode)
            elif component == Component.ENHANCEMENT:
                mode = EnhancementMode.to_enum(mode)
            elif component == Component.SHORTCODE:
                mode = ShortcodeMode.to_enum(mode)

        return mode

    @staticmethod
    def validate_segment_mode_field_for_type(mode: Union[SegmentMode, str, None], type: str):
        """
        Validates if the provided 'mode' parameter is a valid parameter
        for a segment object of the provided 'type'. The 'mode' parameter
        should be a SegmentMode enum or one of its string values to be
        accepted.
        
        This method will raise an Exception if not valid or will return
        it as it is, or as a SegmentMode if one of its valid string
        values provided.
        """
        return ElementValidator.validate_mode_for_type(Component.SEGMENT, mode, type)

    @staticmethod
    def validate_enhancement_mode_field_for_type(mode: Union[EnhancementMode, str, None], type: str):
        """
        Validates if the provided 'mode' parameter is a valid parameter for
        an enhancement object of the provided 'type'. The 'mode' parameter
        should be an EnhancementMode enum or one of its string values to be
        accepted.
        
        This method will raise an Exception if not valid or will return it
        as it is, or as a EnhancementMode if one of its valid string values
        provided.
        """
        return ElementValidator.validate_mode_for_type(Component.ENHANCEMENT, mode, type)

    @staticmethod
    def validate_shortcode_mode_field_for_type(mode: Union[ShortcodeMode, str, None], type: str):
        """
        Validates if the provided 'mode' parameter is a valid parameter for
        a shortcode object of the provided 'type'. The 'mode' parameter
        should be a ShortcodeMode enum or one of its string values to be
        accepted.
        
        This method will raise an Exception if not valid or will return it
        as it is, or as a ShortcodeMode if one of its valid string values
        provided.
        """
        return ElementValidator.validate_mode_for_type(Component.SHORTCODE, mode, type)
    
    @staticmethod
    def validate_mode_for_type(component: Component, mode: Union[str, None], type: str):
        type = ShortcodeType.to_enum(type)

        if component == Component.SEGMENT:
            config = ElementValidator.get_config_as_segment(type)
            mode = ElementValidator.validate_segment_mode_field(mode)
        elif component == Component.ENHANCEMENT:
            config = ElementValidator.get_config_as_enhancement(type)
            mode = ElementValidator.validate_enhancement_mode_field(mode)
        elif component == Component.SHORTCODE:
            config = ElementValidator.get_config_as_shortcode(type)
            mode = ElementValidator.validate_shortcode_mode_field(mode)

        if not mode in config.modes:
            raise Exception(f'The "{type}" type does not accept the provided "{mode}" mode.')
        
        return mode
    
    @staticmethod
    def validate_segment_duration_field(duration: Union[int, float, str, None]):
        """
        Validates that the 'duration' provided, if not None, has a 
        valid and positive numeric value or is a string accepted 
        for a segment.

        This method will raise an Exception if not None value 
        provided but a invalid one.
        """
        return ElementValidator.validate_duration_field(Component.SEGMENT, duration)

    @staticmethod
    def validate_enhancement_duration_field(duration: Union[int, float, str, None]):
        """
        Validates that the 'duration' provided, if not None, has a 
        valid and positive numeric value or is a string accepted 
        for an enhancement.

        This method will raise an Exception if not None value 
        provided but a invalid one.
        """
        return ElementValidator.validate_duration_field(Component.ENHANCEMENT, duration)
    
    @staticmethod
    def validate_shortcode_duration_field(duration: Union[int, float, str, None]):
        """
        Validates that the 'duration' provided, if not None, has a 
        valid and positive numeric value or is a string accepted 
        for a shortcode.

        This method will raise an Exception if not None value 
        provided but a invalid one.
        """
        return ElementValidator.validate_duration_field(Component.SHORTCODE, duration)
    
    @staticmethod
    def validate_duration_field(component: Component, duration: Union[int, float, str, None]):
        if duration is not None:
            if not variable_is_positive_number(duration):
                if component == Component.SEGMENT:
                    duration = SegmentStringDuration.to_enum(duration)
                elif component == Component.ENHANCEMENT:
                    duration = EnhancementStringDuration.to_enum(duration)
                elif component == Component.SHORTCODE:
                    duration = ShortcodeStringDuration.to_enum(duration)

        return duration
    
    @staticmethod
    def validate_segment_duration_field_for_type(duration: Union[SegmentStringDuration, int, float, str, None], type: str):
        """
        Validates if the provided 'duration' parameter is a valid
        parameter for a segment object of the provided 'type'. The
        'duration' parameter should be a SegmentStringDuration enum or one
        of its string values to be accepted.
        
        This method will raise an Exception if not valid or will return it
        as it is, or as a SegmentStringDuration if one of its valid string
        values provided.
        """
        return ElementValidator.validate_duration_for_type(Component.SEGMENT, duration, type)
        
    @staticmethod
    def validate_enhancement_duration_field_for_type(duration: Union[EnhancementStringDuration, int, float, str, None], type: str):
        """
        Validates if the provided 'duration' parameter is a valid
        parameter for an enhancement object of the provided 'type'. The
        'duration' parameter should be a EnhancementStringDuration enum or
        one of its string values to be accepted.
        
        This method will raise an Exception if not valid or will return it
        as it is, or as a EnhancementStringDuration if one of its valid
        string values provided.
        """
        return ElementValidator.validate_duration_for_type(Component.ENHANCEMENT, duration, type)
    
    @staticmethod
    def validate_shortcode_duration_field_for_type(duration: Union[ShortcodeStringDuration, int, float, str, None], type: str):
        """
        Validates if the provided 'duration' parameter is a valid
        parameter for a shortcode object of the provided 'type'. The
        'duration' parameter should be a ShortcodeStringDuration enum or
        one of its string values to be accepted.
        
        This method will raise an Exception if not valid or will return it
        as it is, or as a ShortcodeStringDuration if one of its valid
        string values provided.
        """
        return ElementValidator.validate_duration_for_type(Component.SHORTCODE, duration, type)
    
    @staticmethod
    def validate_duration_for_type(component: Component, duration: Union[SegmentStringDuration, EnhancementStringDuration, ShortcodeStringDuration, int, float, str, None], type: str):
        type = ShortcodeType.to_enum(type)

        if component == Component.SEGMENT:
            config = ElementValidator.get_config_as_segment(type)
            duration = ElementValidator.validate_segment_duration_field(duration)
        elif component == Component.ENHANCEMENT:
            config = ElementValidator.get_config_as_enhancement(type)
            duration = ElementValidator.validate_enhancement_duration_field(duration)
        elif component == Component.SHORTCODE:
            config = ElementValidator.get_config_as_shortcode(type)
            duration = ElementValidator.validate_shortcode_duration_field(duration)

        if not variable_is_positive_number(duration) and not duration in config.string_durations:
            raise Exception(f'The "{type.value}" type does not accept the provided "{duration}" duration.')
        
        return duration
    
    @staticmethod
    def validate_segment_start_field(start: Union[int, float, str, None]):
        return ElementValidator.validate_start_field(Component.SEGMENT, start)
    
    @staticmethod
    def validate_enhancemen_start_field(start: Union[int, float, str, None]):
        return ElementValidator.validate_start_field(Component.ENHANCEMENT, start)
    
    @staticmethod
    def validate_shortcode_start_field(start: Union[int, float, str, None]):
        return ElementValidator.validate_start_field(Component.SHORTCODE, start)

    @staticmethod
    def validate_start_field(component: Component, start: Union[int, float, str, None]):
        if start is not None:
            if not variable_is_positive_number(start):
                if component == Component.SEGMENT:
                    start = SegmentStart.to_enum(start)
                elif component == Component.ENHANCEMENT:
                    start = EnhancementStart.to_enum(start)
                elif component == Component.SHORTCODE:
                    start = ShortcodeStart.to_enum(start)

        return start




    @staticmethod
    def get_config_as_segment(type: str):
        """
        Returns the .config_as_segment if existing or raises an Exception if not.
        """
        config_as_segment = Configuration.get_configuration_by_type(type).config_as_segment
        if not config_as_segment:
            raise Exception(f'The "{type}" type cannot be used as a segment.')
        
        return config_as_segment
    
    @staticmethod
    def get_config_as_enhancement(type: str):
        """
        Returns the .config_as_enhancement if existing or raises an Exception if not.
        """
        config_as_enhancement = Configuration.get_configuration_by_type(type).config_as_enhancement
        if not config_as_enhancement:
            raise Exception(f'The "{type}" type cannot be used as an enhancement.')
        
        return config_as_enhancement
    
    @staticmethod
    def get_config_as_shortcode(type: str):
        """
        Returns the .config_as_shortcode if existing or raises an Exception if not.
        """
        config_as_shortcode = Configuration.get_configuration_by_type(type).config_as_shortcode
        if not config_as_shortcode:
            raise Exception(f'The "{type}" type cannot be used as a shortcode.')
        
        return config_as_shortcode




from yta_general_utils.programming.enum import YTAEnum as Enum


class ShortcodeTagType(Enum):
    """
    This Enum represents the different type of shortcodes that
    we can handle according to their scopes. It could be simple
    [tag] shortcode or a block-scoped one [tag] ... [/tag].
    """
    BLOCK = 'block'
    """
    Shortcode type that is built with a start tag [tag], an
    end tag [/tag] and a content between both of those tags.
    """
    SIMPLE = 'simple'
    """
    Shortcode type that is built with a simple tag [tag].
    """


from youtube_autonomous.elements.builder.element_builder import ElementBuilder
from youtube_autonomous.elements.validator.element_parameter_validator import ElementParameterValidator
from youtube_autonomous.elements.builder.enums import Premade
from yta_general_utils.programming.parameter_obtainer import ParameterObtainer
from moviepy.editor import VideoFileClip


class PremadeElementBuilder(ElementBuilder):
    @classmethod
    def premade_name_to_class(cls, premade_name: str):
        """
        Returns the corresponding premade class according to the
        provided 'premade_name'. If no premade class found, the
        return will be None.
        """
        valid_name = Premade.get_valid_name(premade_name)

        if not valid_name:
            raise Exception(f'The provided premade name "{premade_name}" is not valid. The valid ones are: {Premade.get_all_names_as_str()}')
        
        return Premade[valid_name].value

    @classmethod
    def build_from_enhancement(cls, enhancement: dict):
        premade_name = enhancement.keywords

        parameters_to_ignore = ['self', 'cls', 'args', 'kwargs', 'duration']
        parameters_not_from_extra = ['duration']
        parameters = cls.get_building_parameters(enhancement, parameters_to_ignore, parameters_not_from_extra)

        premade_class = cls.premade_name_to_class(premade_name)

        generated = premade_class().generate(**parameters)

        if isinstance(generated, str):
            # If the premade is generated with manim we will only
            # get the filename as return, so we need it as a clip
            generated = VideoFileClip(generated)

        return generated

    @classmethod
    def build_from_segment(cls, segment: dict):
        premade_name = segment.keywords

        parameters_to_ignore = ['self', 'cls', 'args', 'kwargs', 'duration']
        parameters_not_from_extra = ['duration']
        parameters = cls.get_building_parameters(segment, parameters_to_ignore, parameters_not_from_extra)

        premade_class = cls.premade_name_to_class(premade_name)

        generated = premade_class().generate(**parameters)

        if isinstance(generated, str):
            # If the premade is generated with manim we will only
            # get the filename as return, so we need it as a clip
            generated = VideoFileClip(generated)

        return generated

    @classmethod
    def get_building_parameters(cls, element: dict, parameters_to_ignore: list[str], parameters_not_from_extra: list[str]):
        """
        This method extracts the parameters needed for building the text
        premade that exist in the 'extra_params' field of the provided
        'element' dict, replaces the ones in 'not_extra_parameters' with
        the ones in the main 'element' dict and, finally, keeps the only
        parameters needed to build the effect and returns them. This means
        that only the 'mandatory' or 'optional' parameters to build the
        effect, if existing, will be returned extracted from where it was
        specified.
        """
        premade_name = element.keywords
        
        ElementParameterValidator.validate_premade_name(premade_name)

        premade_class = cls.premade_name_to_class(premade_name)
        parameters = ElementBuilder.extract_extra_params(element, premade_class.generate, parameters_to_ignore)

        # TODO: What if not extra parameter does not exist in main (?)
        for parameter_not_from_extra in parameters_not_from_extra:
            parameters[parameter_not_from_extra] = getattr(element, parameter_not_from_extra)
        
        actual_parameters = ParameterObtainer.get_parameters_from_method(premade_class.generate)

        # We keep only 'mandatory' or 'optional' parameters removing useless
        parameters = {key: value for key, value in parameters.items() if key in actual_parameters['mandatory'] or key in actual_parameters['optional']}

        return parameters


from youtube_autonomous.elements.builder.element_builder import ElementBuilder
from youtube_autonomous.elements.validator.element_parameter_validator import ElementParameterValidator
from youtube_autonomous.elements.builder.enums import EffectPremade
from yta_general_utils.programming.parameter_obtainer import ParameterObtainer
from moviepy.editor import VideoFileClip, CompositeVideoClip, ColorClip, ImageClip, AudioFileClip, AudioClip, CompositeAudioClip
from typing import Union


class EffectElementBuilder(ElementBuilder):
    """
    This builders allows you to generate 'EFFECT' content.
    """
    @classmethod
    def effect_name_to_class(cls, effect_name: str):
        """
        Returns the effect class according to the provided 'effect_name'
        parameter. It will return None if no effect found for that
        'effect_name' parameter.
        """
        valid_name = EffectPremade.get_valid_name(effect_name)

        if not valid_name:
            raise Exception(f'The provided effect premade name "{effect_name}" is not valid. The valid ones are: {EffectPremade.get_all_names_as_str()}')
        
        return EffectPremade[valid_name].value
    
    @classmethod
    def build_from_enhancement(cls, enhancement: dict):
        effect_name = enhancement.keywords
        effect = cls.effect_name_to_class(effect_name)

        return effect

    @classmethod
    def get_building_parameters(cls, element: dict, parameters_to_ignore: list[str], parameters_not_from_extra: list[str]):
        """
        This method extracts the parameters needed for building the effect
        that exist in the 'extra_params' field of the provided 'element'
        dict, replaces the ones in 'not_extra_parameters' with the ones
        in the main 'element' dict and, finally, keeps the only parameters
        needed to build the effect and returns them. This means that only
        the 'mandatory' or 'optional' parameters to build the effect, if
        existing, will be returned extracted from where it was specified.
        """
        effect_name = element.keywords
        
        ElementParameterValidator.validate_effect_name(effect_name)

        effect_class = cls.effect_name_to_class(effect_name)
        parameters = ElementBuilder.extract_extra_params(element, effect_class.apply, parameters_to_ignore)

        # TODO: What if not extra parameter does not exist in main (?)
        for parameter_not_from_extra in parameters_not_from_extra:
            parameters[parameter_not_from_extra] = getattr(element, parameter_not_from_extra)
        
        actual_parameters = ParameterObtainer.get_parameters_from_method(effect_class.apply)

        # We keep only 'mandatory' or 'optional' parameters removing useless
        parameters = {key: value for key, value in parameters.items() if key in actual_parameters['mandatory'] or key in actual_parameters['optional']}

        return parameters


RULES_SUBCLASSES = [
    'AIImageElementRules',
    'AIVideoElementRules',
    'ImageElementRules',
    'VideoElementRules',
    'StockElementRules',
    'CustomStockElementRules',
    'MemeElementRules',
    'SoundElementRules',
    'YoutubeVideoElementRules',
    'TextElementRules',
    'PremadeElementRules',
    'EffectElementRules',
    'GreenscreenElementRules'
]

BUILDER_SUBCLASSES = [
    'ImageElementBuilder',
    'AIImageElementBuilder',
    'VideoElementBuilder',
    'AIVideoElementBuilder',
    'CustomStockElementBuilder',
    'StockElementBuilder',
    'SoundElementBuilder',
    'MemeElementBuilder',
    'YoutubeVideoElementBuilder',
    'TextElementBuilder',
    'PremadeElementBuilder'
]


from youtube_autonomous.elements.builder.element_builder import ElementBuilder
from yta_multimedia.greenscreen.custom.greenscreen import Greenscreen


class GreenscreenElementBuilder(ElementBuilder):
    """
    This builders allows you to generate 'GREENSCREEN' content.
    """
    @classmethod
    def build_from_enhancement(cls, enhancement: dict):
        """
        Returns an ImageGreenscreen or VideoGreenscreen instance
        initialized with the provided filename or url (filename
        has priority) or raises an Exception if something goes
        wrong.
        """
        filename = enhancement.filename
        url = enhancement.url

        if not url and not filename:
            # TODO: Can this happen (?)
            raise Exception('No "url" nor "filename" provided and at least one is needed.')

        if url and not filename:
            return Greenscreen.init(url)

        return Greenscreen.init(filename)


from youtube_autonomous.elements.builder.element_builder import ElementBuilder
from youtube_autonomous.elements.validator.element_parameter_validator import ElementParameterValidator
from youtube_autonomous.elements.builder.enums import TextPremade
from yta_multimedia.video.generation.manim.classes.text.text_word_by_word_manim_animation import TextWordByWordManimAnimation
from yta_general_utils.programming.parameter_obtainer import ParameterObtainer
from moviepy.editor import VideoFileClip


class TextElementBuilder(ElementBuilder):
    @classmethod
    def text_premade_name_to_class(cls, text_premade_name: str):
        """
        Returns the corresponding text premade class according to
        the provided 'text_premade_name'. If no text premade class
        found, the return will be None.
        """
        valid_name = TextPremade.get_valid_name(text_premade_name)

        if not valid_name:
            raise Exception(f'The provided text premade name "{text_premade_name}" is not valid. The valid ones are: {TextPremade.get_all_names_as_str()}')
        
        return TextPremade[valid_name].value

    @classmethod
    def build_from_enhancement(cls, enhancement: dict):
        text_premade_name = enhancement.keywords

        parameters_to_ignore = ['self', 'cls', 'args', 'kwargs', 'output_filename', 'duration', 'text']
        parameters_not_from_extra = ['duration', 'text']
        parameters = cls.get_building_parameters(enhancement, parameters_to_ignore, parameters_not_from_extra)

        return cls.build_custom_from_text_premade_name(text_premade_name, **parameters)

    @classmethod
    def build_from_segment(cls, segment: dict):
        text_premade_name = segment.keywords

        parameters_to_ignore = ['self', 'cls', 'args', 'kwargs', 'output_filename', 'duration', 'text']
        parameters_not_from_extra = ['duration', 'text']
        parameters = cls.get_building_parameters(segment, parameters_to_ignore, parameters_not_from_extra)

        return cls.build_custom_from_text_premade_name(text_premade_name, **parameters)
    
    @classmethod
    def build_custom_from_text_premade_name(cls, text_premade_name, **parameters):
        """
        This method instantiates the 'text_animation_class' Manim
        text animation class and uses the provided 'parameters' to
        build the text animation. The provided 'parameters' must 
        fit the ones requested by the provided class 'generate'
        method.
        """
        ElementParameterValidator.validate_premade_name(text_premade_name)

        text_premade_class = cls.text_premade_name_to_class(text_premade_name)

        # We generate the animation to return it
        filename = text_premade_class().generate(**parameters)

        # TODO: Maybe 'mask' to set background as transparent if available (?)
        return VideoFileClip(filename)
    
    @classmethod
    def get_building_parameters(cls, element: dict, parameters_to_ignore: list[str], parameters_not_from_extra: list[str]):
        """
        This method extracts the parameters needed for building the text
        premade that exist in the 'extra_params' field of the provided
        'element' dict, replaces the ones in 'not_extra_parameters' with
        the ones in the main 'element' dict and, finally, keeps the only
        parameters needed to build the effect and returns them. This means
        that only the 'mandatory' or 'optional' parameters to build the
        effect, if existing, will be returned extracted from where it was
        specified.
        """
        text_premade_name = element.keywords
        
        ElementParameterValidator.validate_text_class_name(text_premade_name)

        text_premade_class = cls.text_premade_name_to_class(text_premade_name)
        parameters = ElementBuilder.extract_extra_params(element, text_premade_class.generate, parameters_to_ignore)

        # TODO: What if not extra parameter does not exist in main (?)
        for parameter_not_from_extra in parameters_not_from_extra:
            parameters[parameter_not_from_extra] = getattr(element, parameter_not_from_extra)
        
        actual_parameters = ParameterObtainer.get_parameters_from_method(text_premade_class.generate)

        # We keep only 'mandatory' or 'optional' parameters removing useless
        parameters = {key: value for key, value in parameters.items() if key in actual_parameters['mandatory'] or key in actual_parameters['optional']}

        return parameters


from yta_multimedia.video.generation.google.google_search import GoogleSearch
from yta_multimedia.video.generation.google.youtube_search import YoutubeSearch
from yta_multimedia.video.generation.manim.classes.video.test_video_manim_animation import TestVideoMobjectIn2DManimAnimation, TestVideoOpenGLMobjectIn2DManimAnimation, TestVideoMobjectIn3DManimAnimation, TestVideoOpenGLMobjectIn3DManimAnimation, TestImageOpenGLMobjectIn3DManimAnimation
from yta_multimedia.video.generation.manim.classes.text.text_triplets_manim_animation import TextTripletsManimAnimation
from yta_multimedia.video.generation.manim.classes.text.text_word_by_word_manim_animation import TextWordByWordManimAnimation
from yta_multimedia.video.generation.manim.classes.text.simple_text_manim_animation import SimpleTextManimAnimation
from yta_multimedia.video.generation.manim.classes.text.rain_of_words_manim_animation import RainOfWordsManimAnimation
from yta_multimedia.video.generation.manim.classes.text.magazine_text_static_manim_animation import MagazineTextStaticManimAnimation
from yta_multimedia.video.generation.manim.classes.text.magazine_text_is_written_manim_animation import MagazineTextIsWrittenManimAnimation
from yta_multimedia.video.edition.effect.color.black_and_white_video_effect import BlackAndWhiteVideoEffect
from yta_multimedia.video.edition.effect.moviepy.blink_video_effect import BlinkVideoEffect
from yta_multimedia.video.edition.effect.moviepy.blur_video_effect import BlurVideoEffect
from yta_multimedia.video.edition.effect.speed.change_speed_effect import ChangeSpeedVideoEffect
from yta_multimedia.video.edition.effect.open_close.fade_in_video_effect import FadeInVideoEffect
from yta_multimedia.video.edition.effect.open_close.fade_out_video_effect import FadeOutVideoEffect
from yta_multimedia.video.edition.effect.display.flip_horizontally_video_effect import FlipHorizontallyVideoEffect
from yta_multimedia.video.edition.effect.display.flip_vertically_video_effect import FlipVerticallyVideoEffect
from yta_multimedia.video.edition.effect.display.multiplied_video_effect import MultipliedVideoEffect
from yta_multimedia.video.edition.effect.custom.photo_video_effect import PhotoVideoEffect
from yta_multimedia.video.edition.effect.speed.reversed_video_effect import ReversedVideoEffect
from yta_multimedia.video.edition.effect.custom.sad_moment_video_effect import SadMomentVideoEffect
from yta_multimedia.video.edition.effect.display.scroll_video_effect import ScrollVideoEffect
from yta_multimedia.video.edition.effect.zoom.linear_zoom_video_effect import LinearZoomVideoEffect
from yta_multimedia.video.edition.effect.speed.stopmotion_video_effect import StopMotionVideoEffect
from yta_multimedia.video.edition.effect.moviepy.position.slide_random_position_moviepy_effect import SlideRandomPositionMoviepyEffect
from yta_multimedia.video.edition.effect.moviepy.position.static.circles_at_position_moviepy_effect import CirclesAtPositionMoviepyEffect
from yta_multimedia.video.edition.effect.moviepy.position.static.stay_at_position_moviey_effect import StayAtPositionMoviepyEffect
from yta_general_utils.programming.enum import YTAEnum as Enum


class Premade(Enum):
    """
    Premade enum class to make our multimedia premades available for the
    app by matching the corresponding class with an Enum variable that is
    used and enabled here.

    This enums are pretended to be matched by their name ignoring cases,
    so feel free to use the 'get_valid_name' YTAEnum method to obtain the
    valid name to be able to intantiate it.
    """
    GOOGLE_SEARCH = GoogleSearch
    YOUTUBE_SEARCH = YoutubeSearch
    # These below are just for testing
    TEST_2D = TestVideoMobjectIn2DManimAnimation
    TEST_2D_OPENGL = TestVideoOpenGLMobjectIn2DManimAnimation
    TEST_3D = TestVideoMobjectIn3DManimAnimation
    TEST_3D_OPENGL = TestVideoOpenGLMobjectIn3DManimAnimation
    TEST_IMAGE_3D_OPENGL = TestImageOpenGLMobjectIn3DManimAnimation

class TextPremade(Enum):
    """
    Text premade enum class to make our multimedia text premades available
    for the app by matching the corresponding class with an Enum variable
    that is used and enabled here.

    This enums are pretended to be matched by their name ignoring cases,
    so feel free to use the 'get_valid_name' YTAEnum method to obtain the
    valid name to be able to intantiate it.
    """
    TRIPLETS = TextTripletsManimAnimation
    WORD_BY_WORD = TextWordByWordManimAnimation
    SIMPLE = SimpleTextManimAnimation
    RAIN_OF_WORDS = RainOfWordsManimAnimation
    MAGAZINE_STATIC = MagazineTextStaticManimAnimation
    MAGAZINE_IS_WRITTEN = MagazineTextIsWrittenManimAnimation

class EffectPremade(Enum):
    """
    Effect premade enum class to make our multimedia effects available for
    the app by matching the corresponding class with an Enum variable that
    is used and enabled here.

    This enums are pretended to be matched by their name ignoring cases,
    so feel free to use the 'get_valid_name' YTAEnum method to obtain the
    valid name to be able to intantiate it.
    """
    BLACK_AND_WHITE = BlackAndWhiteVideoEffect
    BLINK = BlinkVideoEffect
    BLUR = BlurVideoEffect
    CHANGE_SPEED = ChangeSpeedVideoEffect
    FADE_IN = FadeInVideoEffect
    FADE_OUT = FadeOutVideoEffect
    FLIP_HORIZONTALLY = FlipHorizontallyVideoEffect
    FLIP_VERTICALLY = FlipVerticallyVideoEffect
    MULTIPLIED = MultipliedVideoEffect
    PHOTO = PhotoVideoEffect
    REVERSED = ReversedVideoEffect
    SAD_MOMENT = SadMomentVideoEffect
    SCROLL = ScrollVideoEffect
    LINEAR_ZOOM = LinearZoomVideoEffect
    STOP_MOTION = StopMotionVideoEffect
    SLIDE_RANDOM = SlideRandomPositionMoviepyEffect
    # Positioned
    CIRCLES_AT_POSITION = CirclesAtPositionMoviepyEffect
    STAY_AT_POSITION = StayAtPositionMoviepyEffect
